@@ -15,7 +15,7 @@ type rawHTMLParser struct {
 var defaultRawHTMLParser = &rawHTMLParser{}
 
 // NewRawHTMLParser return a new InlineParser that can parse
-// inline htmls
+// inline htmls.
 func NewRawHTMLParser() InlineParser {
 	return defaultRawHTMLParser
 }
@@ -48,57 +48,48 @@ func (s *rawHTMLParser) Parse(parent ast.Node, block text.Reader, pc Context) as
 }
 
 var tagnamePattern = `([A-Za-z][A-Za-z0-9-]*)`
-
-var attributePattern = `(?:[\r\n \t]+[a-zA-Z_:][a-zA-Z0-9:._-]*(?:[\r\n \t]*=[\r\n \t]*(?:[^\"'=<>` + "`" + `\x00-\x20]+|'[^']*'|"[^"]*"))?)`
-var openTagRegexp = regexp.MustCompile("^<" + tagnamePattern + attributePattern + `*[ \t]*/?>`)
-var closeTagRegexp = regexp.MustCompile("^</" + tagnamePattern + `\s*>`)
+var spaceOrOneNewline = `(?:[ \t]|(?:\r\n|\n){0,1})`
+var attributePattern = `(?:[\r\n \t]+[a-zA-Z_:][a-zA-Z0-9:._-]*(?:[\r\n \t]*=[\r\n \t]*(?:[^\"'=<>` + "`" + `\x00-\x20]+|'[^']*'|"[^"]*"))?)` //nolint:golint,lll
+var openTagRegexp = regexp.MustCompile("^<" + tagnamePattern + attributePattern + `*` + spaceOrOneNewline + `*/?>`)
+var closeTagRegexp = regexp.MustCompile("^</" + tagnamePattern + spaceOrOneNewline + `*>`)
 
 var openProcessingInstruction = []byte("<?")
 var closeProcessingInstruction = []byte("?>")
 var openCDATA = []byte("<![CDATA[")
 var closeCDATA = []byte("]]>")
 var closeDecl = []byte(">")
-var emptyComment = []byte("<!---->")
-var invalidComment1 = []byte("<!-->")
-var invalidComment2 = []byte("<!--->")
+var emptyComment1 = []byte("<!-->")
+var emptyComment2 = []byte("<!--->")
 var openComment = []byte("<!--")
 var closeComment = []byte("-->")
-var doubleHyphen = []byte("--")
 
 func (s *rawHTMLParser) parseComment(block text.Reader, pc Context) ast.Node {
 	savedLine, savedSegment := block.Position()
 	node := ast.NewRawHTML()
 	line, segment := block.PeekLine()
-	if bytes.HasPrefix(line, emptyComment) {
-		node.Segments.Append(segment.WithStop(segment.Start + len(emptyComment)))
-		block.Advance(len(emptyComment))
+	if bytes.HasPrefix(line, emptyComment1) {
+		node.Segments.Append(segment.WithStop(segment.Start + len(emptyComment1)))
+		block.Advance(len(emptyComment1))
 		return node
 	}
-	if bytes.HasPrefix(line, invalidComment1) || bytes.HasPrefix(line, invalidComment2) {
-		return nil
+	if bytes.HasPrefix(line, emptyComment2) {
+		node.Segments.Append(segment.WithStop(segment.Start + len(emptyComment2)))
+		block.Advance(len(emptyComment2))
+		return node
 	}
 	offset := len(openComment)
 	line = line[offset:]
 	for {
-		hindex := bytes.Index(line, doubleHyphen)
-		if hindex > -1 {
-			hindex += offset
-		}
-		index := bytes.Index(line, closeComment) + offset
-		if index > -1 && hindex == index {
-			if index == 0 || len(line) < 2 || line[index-offset-1] != '-' {
-				node.Segments.Append(segment.WithStop(segment.Start + index + len(closeComment)))
-				block.Advance(index + len(closeComment))
-				return node
-			}
-		}
-		if hindex > 0 {
-			break
+		index := bytes.Index(line, closeComment)
+		if index > -1 {
+			node.Segments.Append(segment.WithStop(segment.Start + offset + index + len(closeComment)))
+			block.Advance(offset + index + len(closeComment))
+			return node
 		}
+		offset = 0
 		node.Segments.Append(segment)
 		block.AdvanceLine()
 		line, segment = block.PeekLine()
-		offset = 0
 		if line == nil {
 			break
 		}
@@ -153,9 +144,8 @@ func (s *rawHTMLParser) parseMultiLineRegexp(reg *regexp.Regexp, block text.Read
 			if l == eline {
 				block.Advance(end - start)
 				break
-			} else {
-				block.AdvanceLine()
 			}
+			block.AdvanceLine()
 		}
 		return node
 	}
